mod cli;
pub use cli::{
    Action,
    Cli,
    docker_credential_helper,
    exec,
    get,
    GlobalArgs,
    invoke_shortcut,
};

pub(crate) use platform::connect;
pub use platform::server_addr;

pub mod proto;


pub fn show_window(global_args: GlobalArgs) -> anyhow::Result<()> {
    let invoke = cli::InvokeArgs { shortcut_action: proto::ShortcutAction::ShowWindow };
    cli::invoke_shortcut(invoke, global_args)
}


#[cfg(unix)]
mod platform {
    use std::path::PathBuf;
    use tokio::net::UnixStream;

    pub async fn connect(addr: Option<PathBuf>) -> Result<UnixStream, std::io::Error> {
        let default = if cfg!(debug_assertions) { "creddy-server-dev" } else { "creddy-server" };
        let path = addr.unwrap_or_else(|| server_addr(default));
        UnixStream::connect(&path).await
    }

    pub fn server_addr(sock_name: &str) -> PathBuf {
        let mut path = dirs::runtime_dir()
            .unwrap_or_else(|| PathBuf::from("/tmp"));
        if cfg!(debug_assertions) {
            path.push(format!("{sock_name}.dev.sock"))
        }
        else {
            path.push(format!("{sock_name}.sock"));
        }
        path
    }
}


#[cfg(windows)]
mod platform {
    pub fn server_addr(sock_name: &str) -> String {
        if cfg!(debug_assertions) {
            format!(r"\\.\pipe\{sock_name}.dev")
        }
        else {
            format!(r"\\.\pipe\{sock_name}")
        }
    }
}
